# -*- coding: utf-8 -*-
# Copyright 2021 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

"""
The Snmp_server parser templates file. This contains
a list of parser definitions and associated functions that
facilitates both facts gathering and native command generation for
the given network resource.
"""

import re
from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.rm_base.network_template import (
    NetworkTemplate,
)


class Snmp_serverTemplate(NetworkTemplate):
    def __init__(self, lines=None, module=None):
        super(Snmp_serverTemplate, self).__init__(
            lines=lines, tmplt=self, module=module
        )

    # fmt: off
    PARSERS = [
        {
            "name": "enable_traps",
            "getval": re.compile(
                r"""
                ^snmp-server\senable\straps
                (?P<enable>$)?
                (\s(?P<sfp>sfp)\ $)?(\s(?P<poe>poe)$)?(\s(?P<stack>stack)$)?
                (\srmon\s(?P<rising_alarm>rising-alarm)$)?
                (\srmon\s(?P<falling_alarm>falling-alarm)$)?
                (\ssnmp\s(?P<authentication>authentication)$)?
                (\ssnmp\s(?P<linkup>linkup)$)?
                (\ssnmp\s(?P<linkdown>linkdown)$)?
                (\ssnmp\s(?P<coldstart>coldstart)$)?
                (\ssnmp\s(?P<warmstart>warmstart)$)?
                (\s(?P<lldp>lldp)$)?(\s(?P<lldp_med>lldp\smed)$)?
                (\s(?P<storm_control>storm-control)\ $)?
                (\sstp\s(?P<new_root>new-root)$)?
                (\sstp\s(?P<topology_chg>topology-chg)$)?
                (\s(?P<cpu_protect>cpu-protect)$)?
                $""", re.VERBOSE),
            "result": {
                "enable_traps": {
                    "enable": "{{ true if enable is defined }}",
                    "sfp": "{{ true if sfp == 'sfp' else false }}",
                    "poe": "{{ true if poe == 'poe' else false }}",
                    "stack": "{{ true if stack == 'stack' else false }}",
                    "lldp": "{{ true if lldp == 'lldp' else false }}",
                    "lldp_med": "{{ true if lldp_med == 'lldp med' else false }}",
                    "storm_control": "{{ true if storm_control == 'storm-control' else false }}",
                    "stp": {
                        "new_root": "{{ true if new_root == 'new-root' else false }}",
                        "topology_chg": "{{ true if topology_chg == 'topology-chg' else false }}",
                    },
                    "cpu_protect": "{{ true if cpu_protect == 'cpu-protect' else false }}",
                    "rmon": {
                        "rising_alarm": "{{ true if rising_alarm == 'rising-alarm' else false }}",
                        "falling_alarm": "{{ true if falling_alarm == 'falling-alarm' else false }}",
                    },
                    "snmp": {
                        "authentication": "{{ true if authentication == 'authentication' else false }}",
                        "linkup": "{{ true if linkup == 'linkup' else false }}",
                        "linkdown": "{{ true if linkdown == 'linkdown' else false }}",
                        "coldstart": "{{ true if coldstart == 'coldstart' else false }}",
                        "warmstart": "{{ true if warmstart == 'warmstart' else false }}",
                    },
                },
            },
        },
        {
            "name": "enable_traps.environment",
            "getval": re.compile(
                r"""
                ^snmp-server\senable\straps
                (\senvironment\s(?P<environment>\S+))
                $""", re.VERBOSE),
            "result": {
                "enable_traps": {
                    "environment": ["{{ environment }}",],
                },
            },
        },
        {
            "name": "enable",
            "getval": re.compile(
                r"""^snmp-server$""", re.VERBOSE),
            "result": {
                "enable": True,
            },
        },
        {
            "name": "name",
            "getval": re.compile(
                r"""
                ^snmp-server\sname\s(?P<system_name>\S+)
                $""", re.VERBOSE),
            "result": {
                "name": "{{ system_name }}",
            },
        },
        {
            "name": "location",
            "getval": re.compile(
                r"""
                ^snmp-server\slocation\s(?P<system_location>.+)
                $""", re.VERBOSE),
            "result": {
                "location": "{{ system_location }}",
            },
        },
        {
            "name": "contact",
            "getval": re.compile(
                r"""
                ^snmp-server\scontact\s(?P<system_contact>.+)
                $""", re.VERBOSE),
            "result": {
                "contact": "{{ system_contact }}",
            },
        },
        {
            "name": "service_port",
            "getval": re.compile(
                r"""
                ^snmp-server\sservice-port\s(?P<port_number>\d+)
                $""", re.VERBOSE),
            "result": {
                "service_port": "{{ port_number }}",
            },
        },
        {
            "name": "response_broadcast_request",
            "getval": re.compile(
                r"""
                ^snmp-server\sresponse\sbroadcast-request
                $""", re.VERBOSE),
            "result": {
                "response_broadcast_request": True,
            },
        },
        {
            "name": "community",
            "getval": re.compile(
                r"""
                ^snmp-server\scommunity
                (\s(?P<text_type>[0|7]))?
                (\s(?P<community_string>\S+))
                (\sview\s(?P<view_name>\S+))
                (\s(?P<read_mode>(ro|rw)))
                (\saccess\s(?P<ip_acl_list>\S+))?
                """, re.VERBOSE),
            "result": {
                "community": [{
                    "text_type": "{{ text_type }}",
                    "community_string": "{{ community_string }}",
                    "view_name": "{{ view_name }}",
                    "read_mode": "{{ read_mode }}",
                    "ip_acl_list": "{{ ip_acl_list }}",
                },],
            },
        },
        {
            "name": "host",
            "getval": re.compile(
                r"""
                ^snmp-server\shost
                (\s(?P<address>(((\d{1,3}\.){3}\d{1,3})|(([0-9a-fA-F]{0,4}:*)*))))
                (\sversion\s(?P<version>(1|2c|3)))
                (\s(?P<version_3>auth|noauth|priv))?
                (\s(?P<text_type>[0|7]))?
                (\s(?P<community_string>\S+))
                (\sport\s(?P<port>\d+))?
                """, re.VERBOSE),
            "result": {
                "host": [{
                    "address": "{{ address }}",
                    "version": "{{ version }}",
                    "version_3": "{{ version_3 }}",
                    "text_type": "{{ text_type }}",
                    "community_string": "{{ community_string }}",
                    "port": "{{ port }}",
                },],
            },
        },
        {
            "name": "source_interface_traps",
            "getval": re.compile(
                r"""
                ^snmp-server\ssource-interface\straps\s(?P<interfaceid>.+)
                $""", re.VERBOSE),
            "result": {
                "source_interface_traps": "{{ interfaceid }}",
            },
        },
        {
            "name": "user",
            "getval": re.compile(
                r"""
                ^snmp-server\suser
                (\s(?P<user_name>\S+))
                (\s(?P<text_type>[0|7]))?
                (\s(?P<group_name>\S+))
                (\s(?P<v3>v3))?
                (\s(?P<encrypted>encrypted))?
                (\sauth\s(?P<auth>(md5|sha)))?
                (\s(?P<auth_pass>\S+))?
                (\spriv\s(?P<priv_pass>\S+))?
                (\saccess\s(?P<access>\S+))?
                """, re.VERBOSE),
            "result": {
                "user": [{
                    "user_name": "{{ user_name }}",
                    "text_type": "{{ text_type }}",
                    "group_name": "{{ group_name }}",
                    "v3": "{{ true if v3 is defined }}",
                    "encrypted": {
                        "enable": "{{ true if encrypted is defined }}",
                        "auth": "{{ auth }}",
                        "auth_pass": "{{ auth_pass }}",
                        "priv_pass": "{{ priv_pass }}",
                    },
                    "access": "{{ access }}",
                },],
            },
        },
        {
            "name": "group",
            "getval": re.compile(
                r"""
                ^snmp-server\sgroup
                (\s(?P<text_type>[0|7]))?
                (\s(?P<group_name>\S+))
                (\s(?P<security_model>(v1|v2c|v3)))
                (\s(?P<v3>(auth|noauth|priv)))?
                (\sread\s(?P<read>\S+))?
                (\swrite\s(?P<write>\S+))?
                (\snotify\s(?P<notify>\S+))?
                (\saccess\s(?P<access>\S+))?
                $""", re.VERBOSE),
            "result": {
                "group": [{
                    "text_type": "{{ text_type }}",
                    "group_name": "{{ group_name }}",
                    "security_model": "{{ security_model }}",
                    "v3": "{{ v3 }}",
                    "read": "{{ read }}",
                    "write": "{{ write }}",
                    "notify": "{{ notify }}",
                    "access": "{{ access }}",
                },],
            },
        },
        {
            "name": "view",
            "getval": re.compile(
                r"""
                ^snmp-server\sview
                (\s(?P<view_name>\S+))
                (\s(?P<oid_tree>\S+))
                (\s(?P<view_type>included|excluded))
                $""", re.VERBOSE),
            "result": {
                "view": [{
                    "view_name": "{{ view_name }}",
                    "oid_tree": "{{ oid_tree }}",
                    "view_type": "{{ view_type }}",
                },],
            },
        },
        {
            "name": "engineid_local",
            "getval": re.compile(
                r"""
                ^snmp-server\sengineID\slocal
                (\s(?P<engineid_local>[0-9a-fA-F]+))
                $""", re.VERBOSE),
            "result": {
                "engineid_local": "{{ engineid_local }}",
            },
        },
    ]
    # fmt: on
